package	jp.co.sds_corp.util.daemon;

import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* IService のデフォルトの実装を提供します。
* サービス名・状態管理等の実装です。
* Threadの管理はutil.daemon.core.IDaemonクラスの仕事なので
* サービスレベルのクラスは、init(),start()に処理を実装するだけで
* Cron,Timer等の設定を利用できます。
*/
public abstract class AbstractService implements IService, IConfigurable
{
	/** 設定情報 */
	IConfiguration	config;
	/** 中断フラグ */
	boolean	bInterrupt = false;
	/** Validフラグ */
	boolean bValid = true;
	/** debugフラグ */
	boolean debug = false;
	/** サービス名 */
	String	name;
	/** サービスの状態 */
	Status	stat = Status.WAIT;

	/**
	* サービスが割り込まれているかどうかを調べます。
	* @return	サービスが割り込まれている場合はtrue、そうでない場合はfalse
	*/
	public synchronized boolean isInterrupted() {
		return bInterrupt;
	}
	/**
	* サービスの割り込み状態を設定します。
	* @param	var		サービスに割り込む場合はtrue、そうでない場合はfalse
	*/
	public synchronized void setInterrupt( boolean var ) {
		bInterrupt = var;
	}

	/**
	* サービスが実行可能であるかを調べます。
	* @return   サービスが実行可能な場合はtrue、そうでない場合はfalse
	*/
	public synchronized boolean isValid() {
		return bValid;
	}
	/**
	* サービスの実行可能状態を設定します。
	* @param    var     サービスが実行可能な場合はtrue、そうでない場合はfalse
	*/
	public synchronized void setValid( boolean var ) {
		bValid = var;
	}

	/**
	* debugフラグを取得します。
	* @return   debugフラグ
	*/
	public boolean isDebug() {
		return debug;
	}
	/**
	* debugフラグを設定します。
	* @param    var     debugフラグ
	*/
	public void setDebug( boolean var ) {
		debug = var;
	}

	/**
	* サービス名を設定します。
	* @param	var		サービス名
	*/
	public void setName( String var ) {
		name = var;
	}
	/**
	* サービス名を取得します。
	* @return	サービス名
	*/
	public String getName() {
		return name;
	}

	/**
	* サービスの状態を設定します。
	* @param	stat	状態
	*/
	public synchronized void setStatus( Status stat ) {
		this.stat = stat;
	}
	/**
	* サービスの状態を取得します。
	* @return	状態
	*/
	public synchronized Status getStatus() {
		return stat;
	}
	/**
	* サービスの状態を文字列で取得します。
	* @return	状態
	*/
	public String getStatusString() {
		String ret = stat.toString();
		return ret;
	}

	/**
	* サービスを初期化します。
	* このクラスは抽象クラスなので、このメソッドは何もしません。
	*/
	public void init( IConfiguration config )
		throws Exception
	{
		this.config = config;

		String var = config.getAttributes().get( "debug", "false" );
		if ( var.equals( "true" )) {
			setDebug( true );
		}
	}

	/**
	* サービスの文字列表現を返します。
	* @return	サービスの文字列表現
	*/
	public String toString() {
		String ret = getName() + "," + getClass().getName()
						+ ":" + getStatusString();
		if ( isDebug()) {
			ret += " debug=true";
		}
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* ここではデフォルトの内容のみを出力しますので、個別のパラメータを
	* 出力する場合には、サブクラスでこのメソッドをオーバーライドしてください。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		if ( getName() != null ) {
			config.setAttribute( "name", getName() );
			if ( isDebug()) {
				config.setAttribute( "debug", "true" );
			}
		}
		config.setAttribute( "class", getClass().getName() );
	}

}
