package jp.co.sds_corp.util.daemon.core;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.IService;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.daemon.filter.FilterUtility;
import jp.co.sds_corp.util.daemon.filter.IFilter;
import jp.co.sds_corp.util.daemon.filter.TimeFilter;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* デーモンの抽象クラスです。
* デーモン名・状態等を管理します。
* また、Threadを管理して適宜サービスのstart()を呼び出します。
*/
public abstract class AbstractDaemon implements IDaemon, ITerminatable
{
	static CmnLogger logger = CmnLogger.getInstance();
	/** 設定情報 */
	transient private IConfiguration config;
	private Status stat = Status.DEAD;
	private boolean idle = false;
	private boolean debug = false;
	private String name = null;
	private IService svc = null;
	private Vector<IFilter>	vecFilter = new Vector<IFilter>();

	/**
	* デフォルトの初期処理.
	* サービスをロードし、サービスの初期化を行う
	* @param	config	設定情報
	*/
	public void init( IConfiguration config )
		throws Exception
	{
		this.config = config;
		createAllFilter( config );
		String var = config.getAttributes().get( "debug", "false" );
		if ( var.equals( "true" )) {
			setDebug( true );
		}

		IConfiguration  svc_conf = config.getChild( "Service" );
		setService( (IService )ConfigUtility.createInstance( svc_conf ));
		String name = svc_conf.getAttributes().get( "name" );
		getService().setName( name );
		logger.debug( this, "loading service:" + getService() );
		try {
			getService().init( svc_conf );
		} catch ( Throwable ex ) {
			setStatus( Status.ERROR, ex.getMessage() );
			throw( new ServiceInitializeException(
				"service:" + getService() + " initialize error", ex ));
		}
	}

	protected void createAllFilter( IConfiguration config ) throws Exception {
		Iterator<IConfiguration> itr = config.getChilds( "Filter" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			IFilter f = createFilter( c );
			vecFilter.add( f );
		}
		// 設定ファイルの互換性のため
		itr = config.getChilds( "TimeRange" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			TimeFilter tf = new TimeFilter();
			tf.init( c );
			vecFilter.add( tf );
		}
	}

	protected IFilter createFilter( IConfiguration config ) throws Exception {
		IFilter ret;
		ret = (IFilter )ConfigUtility.createInstance( config );
		ret.init( config );
		return ret;
	}

	/**
	* 生存状態の問い合わせ
	* @return	true:生存中 false:死亡
	*/
	public synchronized boolean isAlive() {
		return getStatus() != Status.DEAD;
	}

	/**
	* 状態の取得
	* @return	-1:死亡 1:生存 2:起動中
	*/
	public synchronized Status getStatus()  {
		return stat;
	}
	/**
	* 状態を文字列での取得
	* @return	状態の文字列表現
	*/
	public String getStatusString()  {
		String ret = stat.toString();
		return ret;
	}
	/**
	* 状態の設定
	* @param	var		-1:死亡 1:生存 2:起動中
	*/
	public synchronized void setStatus( Status var )  {
		fireStatusChange( stat, var, "" );
		stat = var;
	}
	/**
	* 状態の設定
	* @param	var		-1:死亡 1:生存 2:起動中
	* @param	msg		メッセージ
	*/
	public synchronized void setStatus( Status var, String msg )  {
		fireStatusChange( stat, var, msg );
		stat = var;
	}

	/**
	* デーモンの名称設定
	* @param	var		デーモンの名称
	*/
	public void setName( String var ) {
		name = var;
	}

	/**
	* デーモンの名称取得
	* @return	デーモンの名称
	*/
	public String getName() {
		return name;
	}

	/**
	* サービスの取得
	* @return	サービス
	*/
	public IService getService() {
		return svc;
	}

	/**
	* サービスの設定
	* @param	var		サービス
	*/
	public void setService( IService var ) {
		svc = var;
	}

	Vector<IStatusChangeListener> vecStat = new Vector<IStatusChangeListener>();
	public void addStatusChangeListener( IStatusChangeListener l ) {
		vecStat.add( l );
	}
	public void removeStatusChangeListener( IStatusChangeListener l ) {
		vecStat.remove( l );
	}
	public void fireStatusChange( Status nOldStat, Status nNewStat, String msg )
	{
		if ( nOldStat == nNewStat ) {
			return;
		}
		StatusChangeEvent ev = new StatusChangeEvent(
								this, nOldStat, nNewStat, msg );
		for ( int i = 0; i < vecStat.size(); i++ ) {
			IStatusChangeListener l = (IStatusChangeListener )vecStat.get( i );
			l.statusChanged( ev );
		}
	}

	/**
	* フィルタを追加します。
	* @param	filter	追加するフィルタ
	*/
	public void addFilter( IFilter filter ) {
		vecFilter.add( filter );
	}
	/**
	* フィルタを削除します。
	* @param	filter	削除するフィルタ
	*/
	public void removeFilter( IFilter filter ) {
		vecFilter.remove( filter );
	}
	/**
	* フィルタの一覧を取得します。
	* @return	フィルタの一覧
	*/
	public List<IFilter> getFilterList() {
		return vecFilter;
	}

	/**
	* このデーモンが実行可能であるかどうかを返します。
	* @return	デーモンが実行可能な場合はtrue、そうでない場合はfalse
	*/
	public boolean isValid() {
		return FilterUtility.isValid( getFilterList() );
	}

	/**
	* debugフラグを設定します。
	* @param	var		debugフラグ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* debugフラグを取得します。
	* @return	debugフラグ
	*/
	public boolean isDebug() {
		return this.debug;
	}

	/**
	* このデーモンの文字列表現
	* @return	このデーモンの文字列表現
	*/
	public String toString() {
		String ret = getName() + "," + getClass().getName();
		for ( int i = 0; i < vecFilter.size(); i++ ) {
			IFilter f = (IFilter )vecFilter.get( i );
			ret += " F{" + f.toString() + "}";
		}
		ret += "[" + getService() + "]";
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* ここではデフォルトの内容のみを出力しますので、個別のパラメータを
	* 出力する場合には、サブクラスでこのメソッドをオーバーライドしてください。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		if ( getName() != null ) {
			config.setAttribute( "name", getName() );
		}
		config.setAttribute( "class", getClass().getName() );
		IConfiguration c;
		for ( int i = 0; i < vecFilter.size(); i++ ) {
			IFilter f = (IFilter )vecFilter.get( i );
			c = config.addChild( "Filter" );
			f.saveConfig( c );
		}

		c = config.addChild( "Service" );
		getService().saveConfig( c );
	}

}

