package	jp.co.sds_corp.util.daemon.file;

import java.io.File;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.daemon.config.ServiceInstantiationException;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* ファイルの受信処理の実装です。
<P>
XMLは以下の形式です。
<PRE>
&lt;Service name="Test" class="jp.co.sds_corp.util.daemon.file.FileChecker"&gt;
	&lt;onError&gt;continue&lt;/onError&gt;
	&lt;FileSet dir="."&gt;
		&lt;Filter class="jp.co.sds_corp.util.daemon.file.RegexpFilenameFilter"&gt;
			&lt;pattern&gt;.*&lt;/pattern&gt;
		&lt;/Filter&gt;
		&lt;Service name="Receiver" class="FileReceiver"&gt;
			...
		&lt;/Service&gt;
	&lt;/FileSet&gt;
&lt;/Service&gt;
</PRE>

&lt;FileSet&gt;要素は複数指定することが可能です。
<P>
&lt;dir&gt;要素で監視するディレクトリを指定します。
<BR>
&lt;Filter&gt;要素で{@link IFilenameFilter}を指定します。
また、ここで指定されたクラスが{@link IConfigurable}をimplementしていれば、
初期化時に{@link IConfigurable#init(IConfiguration)}が呼び出され、
この要素の内容が渡されます。
<BR>
&lt;Service&gt;要素でファイル検出時に呼び出すサービスを指定します。
このサービスは、{@link IFileProcessor}をimplementする必要があります。
*/
public class FileChecker extends AbstractService
		implements FileCheckerMBean
{
	static CmnLogger logger = CmnLogger.getInstance();
	private Vector<IFileProcessor> vecProc = new Vector<IFileProcessor>();
	private int			nOnError = -1;
	private boolean[]	bErr;

	/**
	* このクラスを設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		String var = config.get( "onError", "" );
		if ( var.equals( "break" )) {
			nOnError = 1;
		}
		Iterator<IConfiguration> itr = config.getChilds( "FileSet" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			String className = config.getAttributes().get("class");
			IFileProcessor proc = null;
			IConfiguration svc = c.getChild( "Service" );
			try {
				proc = (IFileProcessor )ConfigUtility.createInstance( svc );
			} catch ( Exception ex ) {
				Exception se = new ServiceInstantiationException(
					"[" + className + "] creation failed.", ex );
				logger.error( this, "error", se );
				continue;
			}
			File dir = new File( c.getAttributes().get( "dir" ));
			if ( !dir.exists()) {
				throw( new Exception( "dir[" + dir + "] is not exists" ));
			}
			proc.setDirectory( dir );
			Iterator itr_flt = c.getChilds( "Filter" );
			while ( itr_flt.hasNext()) {
				IConfiguration flt_conf = (IConfiguration )itr_flt.next();
				String flt_cls = flt_conf.getAttributes().get( "class" );
				Object flt = Class.forName( flt_cls ).newInstance();
				if ( flt instanceof IConfigurable ) {
					((IConfigurable )flt).init( flt_conf );
				}
				proc.addFilter( (IFilenameFilter )flt );
			}

			proc.setName( svc.getAttributes().get( "name" ) );
			proc.init( svc );
			vecProc.add( proc );
		}
		bErr = new boolean[vecProc.size()];
		for ( int i = 0; i < vecProc.size(); i++ ) {
			bErr[i] = false;
		}
	}

	/**
	* ファイルの受信処理を開始します。
	*/
	public void start() {
		setStatus( Status.RUN );
		for ( int i = 0; i < vecProc.size(); i++ ) {
			if ( nOnError == 1 ) {
				if ( bErr[i] ) {
					continue;
				}
			}
			IFileProcessor proc = (IFileProcessor )vecProc.get( i );
			File dir = proc.getDirectory();
			List<IFilenameFilter> lstFlt = proc.listFilter();

			List<File> lstFile = listFiles( dir, lstFlt );
			for ( int j = 0; j < lstFile.size(); j++ ) {
				File file = (File )lstFile.get( j );
				try {
					proc.start( file );
				} catch ( Throwable t ) {
					bErr[i] = true;
					if ( nOnError == 1 ) {
						break;
					}
				}
			}
			try {
				proc.endOfList();
			} catch ( Throwable t ) {
			}
		}
		setStatus( Status.WAIT );
	}

	private List<File> listFiles( File dir, List<IFilenameFilter> lstFlt ) {
		Vector<File> ret = new Vector<File>();
		File[] files = dir.listFiles();
		if ( lstFlt.size() == 0 ) {
			for ( int i = 0; i < files.length; i++ ) {
				ret.add( files[i] );
			}
			return ret;
		}
		Arrays.sort( files, new FileComparator() );

		for ( int i = 0; i < files.length; i++ ) {
			boolean bAcc = false;
			for ( int j = 0; j < lstFlt.size(); j++ ) {
				IFilenameFilter flt = (IFilenameFilter )lstFlt.get( j );
				if ( !flt.accept( dir, files[i].getName() )) {
					bAcc = false;
					break;
				}
				bAcc = true;
			}
			if ( bAcc ) {
				ret.add( files[i] );
			}
		}
		return ret;
	}

	/**
	* ファイルの受信処理を終了します。
	*/
	public void stop() {
		for ( int i = 0; i < vecProc.size(); i++ ) {
			IFileProcessor proc = (IFileProcessor )vecProc.get( i );
			proc.stop();
		}
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		if ( nOnError == 1 ) {
			config.addChild( "onError", "break" );
		}
		for ( int i = 0; i < vecProc.size(); i++ ) {
			IConfiguration c = config.addChild( "FileSet" );
			IFileProcessor proc = (IFileProcessor )vecProc.get( i );
			c.setAttribute( "dir", proc.getDirectory().getPath() );
			List<IFilenameFilter> lstFlt = proc.listFilter();
			for ( int j = 0; j < lstFlt.size(); j++ ) {
				IFilenameFilter flt = (IFilenameFilter )lstFlt.get( j );
				IConfiguration cf = c.addChild( "Filter" );
				flt.saveConfig( cf );
			}
			IConfiguration cp = c.addChild( "Service" );
			proc.saveConfig ( cp );
		}
	}

	/**
	* このサービスの文字列表現を返します。
	*/
	public String toString() {
		String ret = super.toString();
		for ( int i = 0; i < vecProc.size(); i++ ) {
			IFileProcessor proc = (IFileProcessor )vecProc.get( i );
			ret += "\n\t" + proc.getDirectory() + ",";
			List<IFilenameFilter> lstFlt = proc.listFilter();
			for ( int j = 0; j < lstFlt.size(); j++ ) {
				IFilenameFilter flt = (IFilenameFilter )lstFlt.get( j );
				ret += flt.toString() + ",";
			}
			ret += proc.toString();
		}
		return ret;
	}

	private class FileComparator implements java.util.Comparator<File> {
		public int compare( File f1, File f2 ) {
			String fname1 = f1.getPath();
			String fname2 = f2.getPath();
			return ( fname1.compareTo( fname2 ) );
		}
	}

}
