package	jp.co.sds_corp.util.daemon.util;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* syslogを出力するサービスです。
<PRE>
&lt;Service name="Syslog" class="daemon.util.SyslogService"
		facility="USER"
		priority="ERR"
		message="syslog message"
		encode="EUC-JP"&gt;
	&lt;SendTo host="localhost"/&gt;
	&lt;SendTo host="hostA" port="514"/&gt;
&lt;/Service&gt;
</PRE>
*/
public class SyslogService extends AbstractService
{
	static CmnLogger logger = CmnLogger.getInstance();

	class SendTo 
	{
		String	host;
		int		port;
		SendTo( String host, int port ) {
			this.host = host;
			this.port = port;
		}
		/**
		* syslog送信先ホスト名(host)を取得します。
		* @return	hostの内容
		*/
		public String getHost() {
			return host;
		}
		/**
		* syslog送信先ポート番号(port)を取得します。
		* @return	portの内容
		*/
		public int getPort() {
			return port;
		}

	}
	Vector<SendTo>	vecSend = new Vector<SendTo>();
	String	facility;
	String	priority;
	String	message;
	String	encode;

	/**
	* サービスの初期化を行う
	*/
	public void init( IConfiguration config )
		throws Exception
	{
		facility = config.getAttributes().get( "facility" );
		priority = config.getAttributes().get( "priority" );
		message = config.getAttributes().get( "message" );
		encode = config.getAttributes().get( "encode" );

		Iterator<IConfiguration> itr = config.getChilds( "SendTo" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			String host = c.getAttributes().get( "host" );
			int port = c.getAttributes().getInteger( "port", 514 );
			SendTo snd = new SendTo( host, port );
			vecSend.add( snd );
		}
	}

	/**
	* サービスを開始します。
	*/
	public void start() {
		try {
			SyslogMsg msg = new SyslogMsg( getFacility(), getPriority(),
											getMessage() );
			byte[] buf = msg.getData().getBytes( getEncode() );
			DatagramPacket packet = new DatagramPacket( buf, buf.length );
			DatagramSocket sock = new DatagramSocket();
			for ( int i = 0; i < vecSend.size(); i++ ) {
				SendTo snd = (SendTo )vecSend.get( i );
				sock.connect(
					InetAddress.getByName( snd.getHost() ), snd.getPort() );
				sock.send( packet );
				sock.close();
			}
		} catch ( Exception ex ) {
			logger.error( this, "error", ex );
		}
	}

	/**
	* サービスを終了します。
	*/
	public void stop() {
	}

	/**
	* メッセージのfacility(facility)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFacility( String var ) {
		this.facility = var ;
	}
	/**
	* メッセージのfacility(facility)を取得します。
	* @return	facilityの内容
	*/
	public String getFacility() {
		return facility;
	}

	/**
	* メッセージのpriority(priority)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPriority( String var ) {
		this.priority = var ;
	}
	/**
	* メッセージのpriority(priority)を取得します。
	* @return	priorityの内容
	*/
	public String getPriority() {
		return priority;
	}

	/**
	* メッセージ内容(message)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMessage( String var ) {
		this.message = var ;
	}
	/**
	* メッセージ内容(message)を取得します。
	* @return	messageの内容
	*/
	public String getMessage() {
		return message;
	}

	/**
	* 文字コードセット(encode)を設定します。
	* @param	var		設定するデータ
	*/
	public void setEncode( String var ) {
		this.encode = var ;
	}
	/**
	* 文字コードセット(encode)を取得します。
	* @return	encodeの内容
	*/
	public String getEncode() {
		return encode;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.setAttribute( "facility", facility );
		config.setAttribute( "priority", priority );
		config.setAttribute( "message", message );
		for ( int i = 0; i < vecSend.size(); i++ ) {
			SendTo snd = (SendTo )vecSend.get( i );
			IConfiguration c = config.addChild( "SendTo" );
			c.setAttribute( "host", snd.getHost() );
			if ( snd.getPort() != 514 ) {
				c.setAttribute( "port", String.valueOf( snd.getPort()) );
			}
		}
	}

	/**
	* このサービスの文字列表現を返します。
	*/
	public String toString() {
		String ret = super.toString();
		ret += " F=" + facility + " P=" + priority + " M=" + message;
		for ( int i = 0; i < vecSend.size(); i++ ) {
			SendTo snd = (SendTo )vecSend.get( i );
			ret += " to[" + snd.getHost();
			if ( snd.getPort() != 514 ) {
				ret += "," + snd.getPort();
			}
			ret += "]";
		}
		return ret;
	}
}

